/*  
    AVFS: A Virtual File System Library
    Copyright (C) 1998  Miklos Szeredi (mszeredi@inf.bme.hu)
    
    This file can be distributed either under the GNU LGPL, or under
    the GNU GPL. See the file COPYING.LIB and COPYING. 
*/

#ifndef __ARCHIVES_H
#define __ARCHIVES_H

#include "cache.h"

#define KEEP_TIME    300 /* seconds */
#define VBLOCKSIZE    1024
#define DIR_SEP_CHAR '/'

struct arch_entry;

#define INOF_DIRTY    (1 << 0)
#define INOF_CREATED  (1 << 1)

typedef struct arch_inode {
  struct avstat st;
  struct avstat origst;
  char *syml;
  struct arch_entry *subdir;
  int inoflags;

  /* Not used by this module, but almost everything needs these */
  avoff_t offset;
  avoff_t realsize;
  int   typeflag;

  void *udata;
} arch_inode;

#define ARCHE_NOCASE  (1 << 0)

typedef struct arch_entry {
  char *name;
  int entflags;
  struct arch_inode *ino;
  
  struct arch_entry *next;
  struct arch_entry **prevp;
} arch_entry;

struct archive;

#define ARCHF_NOFILE    (1 << 0)
#define ARCHF_WRITABLE  (1 << 1)

typedef struct arch_devd {
  struct vdev_info *vdev;

  int typeflags;
  
  avblksize_t blksize;

  int (*parsefunc)(ave *, vpath *, struct archive *);
  int (*getminor)(ave *, vpath *);
  
  int major;

  int keep_time;
  int valid_time;

  void *udata;
} arch_devd;


typedef struct archive {
  struct arch_inode *root;

  arch_devd *dd;
  cache_obj *cobj;

  int minor;
  avino_t inoctr;
  
  avtime_t mtime;
  avmode_t mode;
  avdev_t  dev;
  avuid_t  uid;
  avgid_t  gid;

  void *udata;
} archive;

typedef struct arch_file {
  int fh;
  avoff_t ptr;
} arch_file;

typedef struct arch_fdi {
  archive *arch;
  struct vdev_info *vdev;
  arch_inode *ino;

  int append;

  /* Only for readdir() */
  arch_entry *ent;
  int loc;

  /* Only for read() */
  arch_file file;       /* The actual "big" archive file */
  avoff_t ptr;          /* Pointer in the inside file */

  avoff_t offset;       /* Offset of the inside file */
  avoff_t size;         /* Actual (compressed) size of the inside file */

  avoff_t lptr;         /* Pointer in the "uncommpressed" file */

  void *udata;          /* User data */
} arch_fdi;


extern arch_inode *__av_make_node(ave *v, struct avstat *initstat);
extern arch_entry *__av_new_entry(ave *v);
extern char       *__av_split_path(ave *v, char *path);
extern arch_entry *__av_find_entry(ave *v, arch_inode *root, const char *path);
extern void        __av_unlink_entry(arch_entry *ent);
extern void        __av_insert_entry(arch_inode *dir, arch_entry *ent);
extern arch_inode *__av_make_dir(ave *v, arch_inode *parentdir,
				 struct avstat *initstat);
extern arch_inode *__av_make_dir_rec(ave *v, archive *arch, arch_inode *root,
				     char *path, int entflags);

extern struct vdev_info *__av_init_arch(ave *v, const char *name);

#endif /* __ARCHIVES_H */
