/*
    AVFS: A Virtual File System Library
    Copyright (C) 1998  Miklos Szeredi (mszeredi@inf.bme.hu)

    This file can be distributed either under the GNU LGPL, or under
    the GNU GPL. See the file COPYING.LIB and COPYING. 
*/

#include "config.h"
#include "parse.h"
#include "vdev.h"

#define VIRTDIRSIZE AVFS_DIRENT_SIZE

#define FVIRTUAL  0x01
#define FINTERNAL 0x02

#define ISVIRTUAL(fd) (__av_priv.file_table[fd].type & FVIRTUAL)
#define FD_OK(fd)      ((fd) >= 0 && (fd) < AVFS_MAX_FILES)
#define IS_LOCAL(fd)   (!__av_priv.inited || !FD_OK(fd) || !ISVIRTUAL(fd))
#define DO_VIRTDIR(fd) (__av_priv.inited && FD_OK(fd) && __av_priv.file_table[fd].virtdir)

#define AVFS_MAX_FILES   1024

typedef struct {
  int type;
  void *devinfo;
  struct vdev_info *vdev;
  vpath *path;
  int oflags;

  int virtdir; /* These are for the 'make pwd work' hack */
  struct avdirent *virtent;
} finfo;


struct avfs_private {
  int inited;
  int vcwd_inited;
  int avfs_flags;
  
  char *tmpdir;
  int tmpdirusage;

  struct vdev_info **vdevs;
  int vdevs_size;
  int vdev_num;

  char *vcwd;
  finfo file_table[AVFS_MAX_FILES];
};

typedef struct {
  int fd;
  int loc;
  int size;
  char *buf;
} AVFS_DIR;

#ifdef HAVE_OPEN64

typedef long long p_ino64_t;
typedef long long p_off64_t;
typedef long long p_blkcnt64_t;

#endif 

extern struct avfs_private __av_priv;

extern void __av_init_vcwd(void);
extern void __av_init_misc(void);
extern void __av_init_modules(void);   
extern int  __av_unlink_recursive(const char *dir);

#define AVLOGSYS(nam, str, i1, i2) \
   AVDEB((AVLOG_SYSCALL, "%s: %s [ 0x%04lx,0x%04lx ]", \
            nam, str != NULL ? str : "(null)", i1, i2))
