/*
    AVFS: A Virtual File System Library
    Copyright (C) 1998  Miklos Szeredi (mszeredi@inf.bme.hu)

    This file can be distributed either under the GNU LGPL, or under
    the GNU GPL. See the file COPYING.LIB and COPYING. 
*/

#include "config.h"

#include "vdev.h"
#include "avfsint.h"

#include <dirent.h>
#include <stdio.h>
#include <string.h>
#include <dlfcn.h>

#define MAX_MODULENAME 32
#define MODULEPREFIX "avfs_module_"
#define MODULEPREFIXLEN 12

static void *load_module(const char *modname)
{
  char *modpath;
  void *lib_handle;

  modpath = __av_malloc(DUMMYV, strlen(MODULEDIR) + 1 + strlen(modname) + 1);
  if(modpath == NULL) return NULL;
  sprintf(modpath, "%s/%s", MODULEDIR, modname);

  lib_handle = dlopen(modpath, RTLD_NOW);
  if(lib_handle == NULL) fprintf(stderr, "avfs: %s\n", dlerror());

  __av_free(modpath);
  
  return lib_handle;
}

void __av_init_modules(void)
{
  DIR *dirp;
  struct dirent *ent;
  char *filename;
  char *mname;
  char modulename[MAX_MODULENAME];
  char miscname[MAX_MODULENAME+64];
  int i;
  void *lib_handle;
  int (*initfunc) (ave *);

  dirp = opendir(MODULEDIR);
  if(dirp == NULL) return;

  while((ent = readdir(dirp)) != NULL) {
    filename = ent->d_name;

    if(strncmp(filename, MODULEPREFIX, MODULEPREFIXLEN) == 0) {
      mname = filename + MODULEPREFIXLEN;
      for(i = 0; mname[i] && mname[i] != '.'; i++);
      if(i > MAX_MODULENAME) continue;

      strncpy(modulename, mname, i);
      modulename[i] = '\0';
      
      sprintf(miscname, "%s.so.%i", modulename, VER_MAJOR);
      if(strcmp(mname, miscname) != 0) continue;

      /* FIXME: Save lib_handle, so that module can be unloaded later */
      lib_handle = load_module(filename);
      if(lib_handle == NULL) continue;

      sprintf(miscname, "__av_init_module_%s", modulename);
      
      initfunc = dlsym(lib_handle, miscname);
      if(initfunc == NULL) {
	dlclose(lib_handle);
	continue;
      }
      
      /* Everything is ready, off we go! */
      (*initfunc)(DUMMYV);
    }
  }
  closedir(dirp);
}
